import 'package:flutter/material.dart';
import 'package:font_awesome_flutter/font_awesome_flutter.dart';
import 'package:flutter_svg/flutter_svg.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_drawer_style.dart';
import '../../components/senior_profile_picture/senior_profile_picture.dart';
import '../../themes/senior_theme.dart';
import '../../themes/senior_theme_provider.dart';

class SeniorDrawer extends StatelessWidget {
  /// Creates an SDS Drawer component.
  ///
  /// The [items] parameter is required.
  const SeniorDrawer({
    Key? key,
    this.footerText,
    required this.items,
    this.profilePicture,
    this.profileSubtitle,
    this.profileTitle,
    this.style,
    this.svgLogoPath,
  }) : super(key: key);

  /// The text displayed in the footer of the drawer.
  final String? footerText;

  /// The list of items in the drawer. Represent the routes to which the user can be redirected.
  final List<SeniorDrawerItem> items;

  /// The profile picture of the user that appears over the list of items.
  final SeniorProfilePicture? profilePicture;

  /// A subtitle that appears next to the user's profile picture.
  final String? profileSubtitle;

  /// A title that appears next to the user's profile picture.
  final String? profileTitle;

  /// The style definitions for the component.
  /// Allows you to configure:
  /// [SeniorDrawerStyle.backgroundColor] the background color of the drawer.
  /// [SeniorDrawerStyle.backIconColor] the color of the drawer back icon.
  /// [SeniorDrawerStyle.boldItemColor] the color of bold type items.
  /// [SeniorDrawerStyle.emphasisItemColor] the color of emphasis-type items.
  /// [SeniorDrawerStyle.footerTextColor] the color of the text in the footer.
  /// [SeniorDrawerStyle.lineColor] the color of the line that separates the profile area and the items list.
  /// [SeniorDrawerStyle.neutralItemColor] the color of neutral-type items.
  /// [SeniorDrawerStyle.profileSubtitleColor] the profile subtitle color.
  /// [SeniorDrawerStyle.profileTitleColor] the profile title color.
  final SeniorDrawerStyle? style;

  /// The svg path of the logo that will be displayed at the top of the drawer.
  /// The file must have the svg extension.
  /// If this parameter is not informed, it will not be displayed immediately.
  final String? svgLogoPath;

  Widget _buildProfile(SeniorTheme? theme, SeniorTheme defaultTheme) {
    final line = Container(
      height: 1.0,
      width: double.infinity,
      color: style?.lineColor ??
          theme?.drawerTheme?.style?.lineColor ??
          defaultTheme.drawerTheme?.style?.lineColor ??
          SeniorColors.secondaryColor200,
    );

    final titleColor = style?.profileTitleColor ??
        theme?.drawerTheme?.style?.profileTitleColor ??
        defaultTheme.drawerTheme?.style?.profileTitleColor ??
        SeniorColors.neutralColor900;

    final subtitleColor = style?.profileSubtitleColor ??
        theme?.drawerTheme?.style?.profileSubtitleColor ??
        defaultTheme.drawerTheme?.style?.profileSubtitleColor ??
        SeniorColors.neutralColor600;

    return profilePicture == null || profileTitle == null
        ? SizedBox.shrink()
        : Column(
            children: [
              Padding(
                padding: const EdgeInsets.symmetric(
                  horizontal: SeniorSpacing.medium,
                  vertical: SeniorSpacing.small,
                ),
                child: Row(
                  children: [
                    profilePicture!,
                    Padding(
                      padding: const EdgeInsets.symmetric(horizontal: SeniorSpacing.small),
                      child: Column(
                        crossAxisAlignment: CrossAxisAlignment.start,
                        children: [
                          Text(
                            profileTitle!,
                            style: SeniorTypography.text(color: titleColor),
                          ),
                          profileSubtitle != null
                              ? Text(
                                  profileSubtitle!,
                                  style: SeniorTypography.small(color: subtitleColor),
                                )
                              : SizedBox.shrink(),
                        ],
                      ),
                    ),
                  ],
                ),
              ),
              line,
            ],
          );
  }

  Widget _buildDrawerHeader(BuildContext context, SeniorTheme? theme, SeniorTheme defaultTheme) {
    return Container(
      padding: const EdgeInsets.only(top: SeniorSpacing.medium),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Padding(
            padding: const EdgeInsets.only(top: SeniorSpacing.small),
            child: Row(
              children: [
                Padding(
                  padding: const EdgeInsets.symmetric(horizontal: SeniorSpacing.small),
                  child: GestureDetector(
                    onTap: () => Navigator.pop(context),
                    child: Padding(
                      padding: const EdgeInsets.all(SeniorSpacing.small),
                      child: FaIcon(
                        FontAwesomeIcons.chevronLeft,
                        color: style?.backIconColor ??
                            theme?.drawerTheme?.style?.backIconColor ??
                            defaultTheme.drawerTheme?.style?.backIconColor ??
                            SeniorColors.pureBlack,
                        size: SeniorTypographyIconSize.title,
                      ),
                    ),
                  ),
                ),
                svgLogoPath != null
                    ? SvgPicture.asset(
                        svgLogoPath!,
                        height: 40.0,
                      )
                    : SizedBox.shrink(),
              ],
            ),
          ),
          _buildProfile(theme, defaultTheme),
        ],
      ),
    );
  }

  Widget _buildItem(SeniorDrawerItem item, BuildContext context, SeniorTheme? theme, SeniorTheme defaultTheme) {
    final Color itemColor;
    final Widget itemTitle;

    switch (item.type) {
      case DrawerItemType.emphasis:
        itemColor = style?.emphasisItemColor ??
            theme?.drawerTheme?.style?.emphasisItemColor ??
            defaultTheme.drawerTheme?.style?.emphasisItemColor ??
            SeniorColors.primaryColor;
        itemTitle = Text(
          item.title,
          style: SeniorTypography.text(color: itemColor),
        );
        break;
      case DrawerItemType.bold:
        itemColor = style?.boldItemColor ??
            theme?.drawerTheme?.style?.boldItemColor ??
            defaultTheme.drawerTheme?.style?.boldItemColor ??
            SeniorColors.neutralColor700;
        itemTitle = Text(
          item.title,
          style: SeniorTypography.titleCardBold(color: itemColor),
        );
        break;
      case DrawerItemType.neutral:
        itemColor = style?.neutralItemColor ??
            theme?.drawerTheme?.style?.neutralItemColor ??
            defaultTheme.drawerTheme?.style?.neutralItemColor ??
            SeniorColors.neutralColor700;
        itemTitle = Text(
          item.title,
          style: SeniorTypography.text(color: itemColor),
        );
        break;
    }

    return ListTile(
      title: Row(
        children: [
          item.icon != null
              ? Padding(
                  padding: const EdgeInsets.only(right: SeniorSpacing.small),
                  child: FaIcon(
                    item.icon,
                    color: itemColor,
                    size: SeniorTypographyIconSize.text,
                  ),
                )
              : SizedBox.shrink(),
          itemTitle,
        ],
      ),
      onTap: () {
        item.onTap();
        Navigator.pop(context);
      },
    );
  }

  Widget _buildDrawerFooter(SeniorTheme? theme, SeniorTheme defaultTheme) {
    final textColor = style?.footerTextColor ??
        theme?.drawerTheme?.style?.footerTextColor ??
        defaultTheme.drawerTheme?.style?.footerTextColor ??
        SeniorColors.neutralColor500;

    if (footerText == null) {
      return SizedBox.shrink();
    } else {
      return Padding(
        padding: const EdgeInsets.symmetric(vertical: SeniorSpacing.small),
        child: Center(
          child: Text(
            footerText!,
            style: SeniorTypography.helper(color: textColor),
          ),
        ),
      );
    }
  }

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    final list = items.map((item) => _buildItem(item, context, theme, defaultTheme)).toList();
    return Container(
      decoration: BoxDecoration(
        color: style?.backgroundColor ??
            theme?.drawerTheme?.style?.backgroundColor ??
            defaultTheme.drawerTheme?.style?.backgroundColor ??
            SeniorColors.pureWhite,
      ),
      width: MediaQuery.of(context).size.width,
      child: ListView(
        children: [
          _buildDrawerHeader(context, theme, defaultTheme),
          Container(
            padding: const EdgeInsets.symmetric(vertical: SeniorSpacing.medium),
            child: Column(
              children: list,
            ),
          ),
          _buildDrawerFooter(theme, defaultTheme),
        ],
      ),
    );
  }
}

class SeniorDrawerItem {
  /// The information needed to create an item from the drawer.
  ///
  /// The [onTap] and [title] parameters are required.
  const SeniorDrawerItem({
    this.icon,
    required this.onTap,
    required this.title,
    this.type = DrawerItemType.neutral,
  });

  /// An icon that will appear next to the item's title.
  final IconData? icon;

  /// Callback function that will be executed when the item is tapped.
  final VoidCallback onTap;

  /// The item's title.
  final String title;

  /// The item type. It can be [DrawerItemType.emphasis], [DrawerItemType.bold] and [DrawerItemType.neutral].
  final DrawerItemType type;
}

/// The possible types for the items in the drawer. It can be [DrawerItemType.emphasis], [DrawerItemType.bold] and
/// [DrawerItemType.neutral].
enum DrawerItemType {
  emphasis,
  bold,
  neutral,
}
