import 'package:flutter/material.dart';
import 'package:pin_code_text_field/pin_code_text_field.dart';

import 'package:senior_design_tokens/senior_design_tokens.dart';

import './senior_pin_code_field_style.dart';
import '../../themes/senior_theme.dart';
import '../../themes/senior_theme_provider.dart';

class SeniorPinCodeField extends StatelessWidget {
  /// Creates a Pin Code Field component.
  /// The [length] parameter is mandatory.
  const SeniorPinCodeField({
    Key? key,
    this.autofocus = false,
    this.controller,
    this.disabled = false,
    this.focusNode,
    this.hasError = false,
    this.hideCaracter = false,
    this.keyboardType = TextInputType.number,
    required this.length,
    this.onDone,
    this.onTextChanged,
    this.style,
    this.wrapAlignment = WrapAlignment.start,
  }) : super(key: key);

  /// Whether the field will automatically receive focus.
  ///
  /// The default value is false.
  final bool autofocus;

  /// The controller for the text field.
  final TextEditingController? controller;

  /// Whether the field will be disabled.
  ///
  /// The default value is false.
  final bool disabled;

  /// The focus control for the component.
  final FocusNode? focusNode;

  /// Whether the field should assume the error state.
  /// The component does not work with validators, but it can display the error status from this parameter.
  ///
  /// The default value is false.
  final bool hasError;

  /// Defines whether the typed content will be omitted from the view.
  /// It will be replaced by a special character used only to demonstrate that the field is filled.
  ///
  /// The default value is false.
  final bool hideCaracter;

  /// The keyboard type for the fields.
  ///
  /// The default value is [TextInputType.number].
  final TextInputType keyboardType;

  /// The number of fields/characters supported by the component.
  final int length;

  /// A callback function executed when all fields are filled.
  final void Function(String)? onDone;

  /// A callback function executed when the contents of a field change.
  final dynamic Function(String)? onTextChanged;

  /// The component's style definitions.
  /// Allows you to configure:
  /// [SeniorPinCodeFieldStyle.defaultBorderColor] the default color for the border color. The color displayed when the
  /// field is not in focus, has no content, or in an no-error state.
  /// [SeniorPinCodeFieldStyle.disabledDefaultBorderColor] the default color for the border color. The color displayed
  /// when the field has no content and is disabled.
  /// [SeniorPinCodeFieldStyle.disabledHasTextBorderColor] the border color for when the field has content and is disabled.
  /// [SeniorPinCodeFieldStyle.disabledPinBoxColor] the field text color for when it is disabled.
  /// [SeniorPinCodeFieldStyle.disabledPinTextColor] the field's background color for when it is disabled.
  /// [SeniorPinCodeFieldStyle.errorBorderColor] the border color for fields that are in an error state.
  /// [SeniorPinCodeFieldStyle.hasTextBorderColor] the border color for when the field has content.
  /// [SeniorPinCodeFieldStyle.highlightColor] the border color for when the field is in focus.
  /// [SeniorPinCodeFieldStyle.pinBoxColor] the field's background color.
  /// [SeniorPinCodeFieldStyle.pinTextColor] the text color of the field.
  final SeniorPinCodeFieldStyle? style;

  /// Defines how the behavior will be when there is a need to break the content on more than one line.
  ///
  /// The default value is [WrapAlignment.start].
  final WrapAlignment wrapAlignment;

  @override
  Widget build(BuildContext context) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme = SeniorThemeProvider.getDefaultTheme(context);

    final pinCodeFields = PinCodeTextField(
      autofocus: autofocus,
      controller: controller,
      defaultBorderColor: disabled
          ? style?.disabledDefaultBorderColor ??
              theme?.pinCodeFieldTheme?.style?.disabledDefaultBorderColor ??
              defaultTheme.pinCodeFieldTheme?.style?.disabledDefaultBorderColor ??
              SeniorColors.secondaryColor300
          : style?.defaultBorderColor ??
              theme?.pinCodeFieldTheme?.style?.defaultBorderColor ??
              defaultTheme.pinCodeFieldTheme?.style?.defaultBorderColor ??
              SeniorColors.secondaryColor500,
      errorBorderColor: style?.errorBorderColor ??
          theme?.pinCodeFieldTheme?.style?.errorBorderColor ??
          defaultTheme.pinCodeFieldTheme?.style?.errorBorderColor ??
          SeniorColors.manchesterColorRed500,
      focusNode: focusNode,
      hasError: hasError,
      hasTextBorderColor: disabled
          ? style?.disabledHasTextBorderColor ??
              theme?.pinCodeFieldTheme?.style?.disabledHasTextBorderColor ??
              defaultTheme.pinCodeFieldTheme?.style?.disabledHasTextBorderColor ??
              SeniorColors.secondaryColor300
          : style?.hasTextBorderColor ??
              theme?.pinCodeFieldTheme?.style?.hasTextBorderColor ??
              defaultTheme.pinCodeFieldTheme?.style?.hasTextBorderColor ??
              SeniorColors.secondaryColor500,
      hideCharacter: hideCaracter,
      highlight: true,
      highlightColor: style?.highlightColor ??
          theme?.pinCodeFieldTheme?.style?.highlightColor ??
          defaultTheme.pinCodeFieldTheme?.style?.highlightColor ??
          SeniorColors.primaryColor500,
      keyboardType: keyboardType,
      maxLength: length,
      onDone: onDone,
      onTextChanged: onTextChanged,
      pinBoxBorderWidth: 1.0,
      pinBoxColor: disabled
          ? style?.disabledPinBoxColor ??
              theme?.pinCodeFieldTheme?.style?.disabledPinBoxColor ??
              defaultTheme.pinCodeFieldTheme?.style?.disabledPinBoxColor ??
              SeniorColors.neutralColor100
          : style?.pinBoxColor ??
              theme?.pinCodeFieldTheme?.style?.pinBoxColor ??
              defaultTheme.pinCodeFieldTheme?.style?.pinBoxColor ??
              SeniorColors.pureWhite,
      pinBoxHeight: 56.0,
      pinBoxOuterPadding: const EdgeInsets.all(SeniorSpacing.xsmall),
      pinBoxRadius: SeniorRadius.medium,
      pinBoxWidth: 40.0,
      pinTextStyle: SeniorTypography.h3(
        color: disabled
            ? style?.disabledPinTextColor ??
                theme?.pinCodeFieldTheme?.style?.disabledPinTextColor ??
                defaultTheme.pinCodeFieldTheme?.style?.disabledPinTextColor ??
                SeniorColors.neutralColor500
            : style?.pinTextColor ??
                theme?.pinCodeFieldTheme?.style?.pinTextColor ??
                defaultTheme.pinCodeFieldTheme?.style?.pinTextColor ??
                SeniorColors.neutralColor800,
      ),
      wrapAlignment: wrapAlignment,
    );

    return disabled
        ? AbsorbPointer(
            child: pinCodeFields,
          )
        : pinCodeFields;
  }
}
